﻿'# Title    : Simple ColorShader
'# Author   : Sascha Bajus
'# Created  : 04/2009
'# Copyright © 2009 Sascha Bajus



#Region "Copyright© Information (German)"
' Diese Software wird bereitgestellt "wie sie ist", ohne jegliche ausdrückliche oder 
' stillschweigende Gewährleistung. In keinem Fall kann der Author haftbar gemacht werden
' für Schäden, die aus der Nutzung dieser Software entstanden sind.
'
' Es ist erlaubt, diese Software für jeden Zweck zu nutzen, einschließlich der kommerziellen 
' Anwendungen, und sie zu verändern, vorbehaltlich der folgenden Einschränkungen: 
'
' 1. Der Ursprung dieser Software darf nicht falsch dargestellt werden; Sie dürfen nicht behaupten, 
'    dass Sie der Urheber der Original-Software sind. Wenn Sie diese Software in einem Produkt 

' 2. Altered-Source-Versionen müssen klar als solche gekennzeichnet, und dürfen nicht als 
'    Original-Software dargestellt werden.
'
' 3. Diese Informationen dürfen nicht entfernt oder geändert werden.
#End Region
#Region "Copyright© Information (English)"
' This software is provided 'as-is', without any express or implied warranty. In no event will 
' the authors be held liable for any damages arising from the use of this software.
'
' Permission is granted to anyone to use this software for any purpose, including commercial 
' applications, and to alter it and redistribute it freely, subject to the following restrictions:
'
' 1. The origin of this software must not be misrepresented; you must not claim that you wrote 
'    the original software. If you use this software in a product, an acknowledgment in the 
'    product documentation would be appreciated but is not required.
'
' 2. Altered source versions must be plainly marked as such, and must not be misrepresented 
'    as being the original software.
'
' 3. This notice may not be removed or altered from any source distribution.
'
#End Region

Imports SlimDX
Imports SlimDX.Direct3D9
Imports SlimDX.DirectInput
Imports sdxBaseEngine


Public Class Project : Inherits SDX9BaseEngine

    Private _vbuffer As VertexBuffer            '# VertexBuffer for TriangleData
    Private _vertexdec As VertexDeclaration     '# DX9 Vertexdeclaration
    Private _useshader As Boolean               '# Use Shader or not
    Private _shader As SlimDX.Direct3D9.Effect  '# Shader Effect

    Public Sub New()

        AddHandler dxRenderScene, AddressOf render
        AddHandler dxKeyboardState, AddressOf keyboard
        AddHandler dxMouseState, AddressOf mouse
        AddHandler dxLoadObjectsSettings, AddressOf LoadObjectsSettings
        ' AddHandler dxLoadObjectsTextures, AddressOf LoadObjectsTextures
        AddHandler dxEngineClosed, AddressOf ExitAplication
        '---------------------------------------------

        If Me.dxEngine("SlimDX - BaseEngine - Simple Vertexshader") = False Then CloseBaseEngine()

    End Sub

#Region "Loading"
    Private Sub LoadObjectsSettings()

        '#create Vertexdata as PositionColored
        Dim vertex(2) As VertexPositionColor
        vertex(0) = New VertexPositionColor(New Vector3(0, 2, 0), Color.Black.ToArgb)
        vertex(1) = New VertexPositionColor(New Vector3(-2.5, -2, 0), Color.Black.ToArgb)
        vertex(2) = New VertexPositionColor(New Vector3(2.5, -2, 0), Color.Black.ToArgb)

        '#write Vertexdata as stream to new VertexBuffer
        _vbuffer = New VertexBuffer(dxDevice, VertexPositionColor.SizeOfBytes * 3, Usage.WriteOnly, VertexFormat.None, Pool.Managed)
        Dim stream As DataStream = _vbuffer.Lock(0, 0, LockFlags.Discard)
        stream.WriteRange(vertex)
        stream.Close()
        stream.Dispose()
        _vbuffer.Unlock()

        '#create vertexdeclaration
        Dim elements(2) As VertexElement
        elements(0) = New VertexElement(0, 0, DeclarationType.Float3, DeclarationMethod.Default, DeclarationUsage.Position, 0)
        elements(1) = New VertexElement(0, 12, DeclarationType.Color, DeclarationMethod.Default, DeclarationUsage.Color, 0)
        elements(2) = VertexElement.VertexDeclarationEnd
        _vertexdec = New VertexDeclaration(dxDevice, elements)


        '#load Shader as Effect
        _shader = Direct3D9.Effect.FromFile(dxDevice, "data\Color.fx", ShaderFlags.PartialPrecision)
        '#enable Technique
        _shader.Technique = "colored"

        '#set camera position
        dxCamera.currentPosition = New Vector3(0, 0, -9)


        dxEngine.TargetBufferColor = Color.CornflowerBlue

    End Sub
#End Region


#Region "## render ##"
    Private Sub render()

        '#use color only
        dxTexture.resetTexture(0)
        '#rotate triangle
        dxTransformWorld = dxMatrix.TimebasedRotationY(0.001)
        '#disable culling
        dxDevice.SetRenderState(RenderState.CullMode, Cull.None)

        If _useshader Then

            '#set shader
            _shader.Begin()
            _shader.SetValue("WorldViewProj", dxWorld.WorldViewProjection) '#set WorldViewProjection
            _shader.SetValue("r", CSng(dxGametime.Timefading(0.001)))   '#set ColorValue for R
            _shader.SetValue("g", CSng(dxGametime.Timefading(0.0005)))   '#set ColorValue for G
            _shader.SetValue("b", CSng(dxGametime.Timefading(0.0008)))  '#set ColorValue for B
            _shader.BeginPass(0)

            '#Draw
            dxDevice.SetStreamSource(0, _vbuffer, 0, VertexPositionColor.SizeOfBytes)
            dxDevice.VertexDeclaration = _vertexdec
            dxDevice.DrawPrimitives(PrimitiveType.TriangleList, 0, 1)

            _shader.EndPass()
            _shader.End()

        Else

            '#Draw without Shader
            dxDevice.SetStreamSource(0, _vbuffer, 0, VertexPositionColor.SizeOfBytes)
            dxDevice.VertexDeclaration = _vertexdec
            dxDevice.DrawPrimitives(PrimitiveType.TriangleList, 0, 1)

        End If

        '#render FPS
        dx2DText.DrawFPS(vbCrLf & "Press Space" & vbCrLf & "Vertexshader: " & _useshader.ToString)
    End Sub
#End Region


#Region "## Inputdevices ##"
    Private Sub keyboard()
        If dxEngine.KeyPressed(Key.Escape) Then CloseBaseEngine()
        If dxEngine.KeyAsSwitch(Key.Space) Then _useshader = Not _useshader
    End Sub

    Private Sub mouse()
    End Sub
#End Region
#Region "## Exit Application ##"

    Private Sub ExitAplication()
        '#clean up
        If Not _vbuffer Is Nothing Then _vbuffer.Dispose()
        If Not _shader Is Nothing Then _shader.Dispose()
        If Not _vertexdec Is Nothing Then _vertexdec.Dispose()
        End
    End Sub
#End Region

    Private Sub InitializeComponent()
        Dim resources As System.ComponentModel.ComponentResourceManager = New System.ComponentModel.ComponentResourceManager(GetType(Project))
        Me.SuspendLayout()
        '
        'Project
        '
        Me.ClientSize = New System.Drawing.Size(284, 262)
        Me.Icon = CType(resources.GetObject("$this.Icon"), System.Drawing.Icon)
        Me.Name = "Project"
        Me.ResumeLayout(False)

    End Sub
End Class

